#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#include <platform/mt_i2c.h>
#include <platform/mt_pmic.h>
#include <string.h>
#else
#include <linux/string.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <linux/pinctrl/consumer.h>
#include <linux/of_gpio.h>
#include <asm-generic/gpio.h>

#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/mm_types.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>

#ifdef CONFIG_OF
#include <linux/of.h>
#include <linux/of_irq.h>
#include <linux/of_address.h>
#include <linux/of_device.h>
#include <linux/regulator/consumer.h>
#include <linux/clk.h>
#endif
#endif

#include "lcm_drv.h"
#include "ddp_irq.h"

#ifdef BUILD_LK

#ifdef GPIO_LCM_PWR
#define GPIO_LCD_PWR      GPIO_LCM_PWR
#else
#define GPIO_LCD_PWR      0xFFFFFFFF
#endif

#ifdef GPIO_LCM_RST
#define GPIO_LCD_RST      GPIO_LCM_RST
#else
#define GPIO_LCD_RST      0xFFFFFFFF
#endif

#define GPIO_LCD_SPI_MI  (GPIO55 | 0x80000000)
#define GPIO_LCD_SPI_MO  (GPIO56 | 0x80000000)
#define GPIO_LCD_SPI_CS  (GPIO53 | 0x80000000)
#define GPIO_LCD_SPI_CK  (GPIO54 | 0x80000000)


static void lcm_set_gpio_output(unsigned int GPIO, unsigned int output)
{
	mt_set_gpio_mode(GPIO, GPIO_MODE_00);
	mt_set_gpio_dir(GPIO, GPIO_DIR_OUT);
	mt_set_gpio_out(GPIO, (output > 0) ? GPIO_OUT_ONE : GPIO_OUT_ZERO);
}
#else

static struct regulator *lcm_vgp;

//static unsigned int GPIO_LCD_PWR_EN;
static unsigned int GPIO_LCD_RST_EN;

static unsigned int GPIO_LCD_SPI_MI;
static unsigned int GPIO_LCD_SPI_MO;
static unsigned int GPIO_LCD_SPI_CS;
static unsigned int GPIO_LCD_SPI_CK;

static void lcm_set_gpio_output(unsigned int GPIO, unsigned int output);
void lcm_get_gpio_infor(void)
{
	static struct device_node *node;

	node = of_find_compatible_node(NULL, NULL, "mediatek,lcm");

//	GPIO_LCD_PWR_EN = of_get_named_gpio(node, "lcm_power_gpio", 0);
	GPIO_LCD_RST_EN = of_get_named_gpio(node, "lcm_reset_gpio", 0);
	pr_debug("LCM: lcm_get_gpio_info: reset %d\n", GPIO_LCD_RST_EN);
	GPIO_LCD_SPI_MI = of_get_named_gpio(node, "lcm_spi_mi_gpio", 0);
	GPIO_LCD_SPI_MO = of_get_named_gpio(node, "lcm_spi_mo_gpio", 0);
	GPIO_LCD_SPI_CS = of_get_named_gpio(node, "lcm_spi_cs_gpio", 0);
	GPIO_LCD_SPI_CK = of_get_named_gpio(node, "lcm_spi_ck_gpio", 0);

	lcm_set_gpio_output(GPIO_LCD_SPI_MI, 1);
	lcm_set_gpio_output(GPIO_LCD_SPI_MO, 1);
	lcm_set_gpio_output(GPIO_LCD_SPI_CS, 1);
	lcm_set_gpio_output(GPIO_LCD_SPI_CK, 1);
}

static void lcm_set_gpio_output(unsigned int GPIO, unsigned int output)
{
	gpio_direction_output(GPIO, output);
	gpio_set_value(GPIO, output);
}


/* get LDO supply */
static int lcm_get_vgp_supply(struct device *dev)
{
	int ret;
	struct regulator *lcm_vgp_ldo;

	pr_debug("LCM: lcm_get_vgp_supply is going\n");

	lcm_vgp_ldo = devm_regulator_get(dev, "reg-lcm");
	if (IS_ERR(lcm_vgp_ldo)) {
		ret = PTR_ERR(lcm_vgp_ldo);
		dev_err(dev, "failed to get reg-lcm LDO, %d\n", ret);
		return ret;
	}

	pr_debug("LCM: lcm get supply ok.\n");

	ret = regulator_enable(lcm_vgp_ldo);
	/* get current voltage settings */
	ret = regulator_get_voltage(lcm_vgp_ldo);
	pr_debug("lcm LDO voltage = %d in LK stage\n", ret);

	lcm_vgp = lcm_vgp_ldo;

	return ret;
}

int lcm_vgp_supply_enable(void)
{
	int ret;
	unsigned int volt;

	pr_debug("LCM: lcm_vgp_supply_enable\n");

	if (NULL == lcm_vgp)
		return 0;

	pr_debug("LCM: set regulator voltage lcm_vgp voltage to 1.8V\n");
	/* set voltage to 1.8V */
	ret = regulator_set_voltage(lcm_vgp, 3300000, 3300000);
	if (ret != 0) {
		pr_err("LCM: lcm failed to set lcm_vgp voltage: %d\n", ret);
		return ret;
	}

	/* get voltage settings again */
	volt = regulator_get_voltage(lcm_vgp);
	if (volt == 3300000)
		pr_err("LCM: check regulator voltage=3300000 pass!\n");
	else
		pr_err("LCM: check regulator voltage=3300000 fail! (voltage: %d)\n", volt);

	ret = regulator_enable(lcm_vgp);
	if (ret != 0) {
		pr_err("LCM: Failed to enable lcm_vgp: %d\n", ret);
		return ret;
	}

	return ret;
}

int lcm_vgp_supply_disable(void)
{
	int ret = 0;
	unsigned int isenable;

	if (NULL == lcm_vgp)
		return 0;

	/* disable regulator */
	isenable = regulator_is_enabled(lcm_vgp);

	pr_debug("LCM: lcm query regulator enable status[0x%d]\n", isenable);

	if (isenable) {
		ret = regulator_disable(lcm_vgp);
		if (ret != 0) {
			pr_err("LCM: lcm failed to disable lcm_vgp: %d\n", ret);
			return ret;
		}
		/* verify */
		isenable = regulator_is_enabled(lcm_vgp);
		if (!isenable)
			pr_err("LCM: lcm regulator disable pass\n");
	}

	return ret;
}

static int lcm_probe(struct device *dev)
{
	pr_debug("LCM: %s enter\n", __func__);
	lcm_get_vgp_supply(dev);
	lcm_get_gpio_infor();

	return 0;
}

static const struct of_device_id lcm_of_ids[] = {
	{.compatible = "mediatek,lcm",},
	{}
};

static struct platform_driver lcm_driver = {
	.driver = {
		   .name = "mtk_lcm",
		   .owner = THIS_MODULE,
		   .probe = lcm_probe,
#ifdef CONFIG_OF
		   .of_match_table = lcm_of_ids,
#endif
		   },
};

static int __init lcm_init(void)
{
	pr_notice("LCM: Register lcm driver\n");
	if (platform_driver_register(&lcm_driver)) {
		pr_err("LCM: failed to register disp driver\n");
		return -ENODEV;
	}

	return 0;
}

static void __exit lcm_exit(void)
{
	platform_driver_unregister(&lcm_driver);
	pr_notice("LCM: Unregister lcm driver done\n");
}
late_initcall(lcm_init);
module_exit(lcm_exit);
MODULE_AUTHOR("mediatek");
MODULE_DESCRIPTION("Display subsystem Driver");
MODULE_LICENSE("GPL");
#endif
/* --------------------------------------------------------------------------- */
/* Local Constants */
/* --------------------------------------------------------------------------- */

#define FRAME_WIDTH  (240)
#define FRAME_HEIGHT (320)

#define HSYNC_PULSE_WIDTH 10
#define HSYNC_BACK_PORCH  20
#define HSYNC_FRONT_PORCH 38
#define VSYNC_PULSE_WIDTH 4
#define VSYNC_BACK_PORCH  4
#define VSYNC_FRONT_PORCH 8


/* --------------------------------------------------------------------------- */
/* Local Variables */
/* --------------------------------------------------------------------------- */

static LCM_UTIL_FUNCS lcm_util = { 0 };

#define SET_RESET_PIN(v)    (lcm_util.set_reset_pin((v)))

#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))

#define spi_set_cs(v) (lcm_set_gpio_output(GPIO_LCD_SPI_CS, v))
#define spi_set_clk(v) (lcm_set_gpio_output(GPIO_LCD_SPI_CK, v))
#define spi_set_sdi(v) (lcm_set_gpio_output(GPIO_LCD_SPI_MO, v))
#define spi_set_sdo(v) (lcm_set_gpio_output(GPIO_LCD_SPI_MI, v))

static void SPI_WriteComm(u8 cmd)
{
	int i;

	spi_set_cs(0);
	UDELAY(10);
	spi_set_clk(0);
	spi_set_sdi(0);
	UDELAY(100);
	spi_set_clk(1);
	UDELAY(100);

	for (i = 0; i < 8; i++) {
		spi_set_clk(0);
		if (cmd & 0x80)
			spi_set_sdi(1);
		else
			spi_set_sdi(0);
		UDELAY(100);
		spi_set_clk(1);
		UDELAY(100);
		cmd <<= 1;
	}
	spi_set_cs(1);
	UDELAY(10);
}

static void SPI_WriteData(u8 data)
{
	int i;

	spi_set_cs(0);
	UDELAY(10);
	spi_set_clk(0);
	spi_set_sdi(1);
	UDELAY(100);
	spi_set_clk(1);
	UDELAY(100);

	for (i = 0; i < 8; i++) {
		spi_set_clk(0);
		if (data & 0x80)
			spi_set_sdi(1);
		else
			spi_set_sdi(0);
		UDELAY(100);
		spi_set_clk(1);
		UDELAY(100);
		data <<= 1;
	}
	spi_set_cs(1);
	UDELAY(10);
}

static void SPI_WRITE_CMD(unsigned char cmd)
{
	SPI_WriteComm(cmd);
}

static void SPI_WRITE_DATA(unsigned char data)
{
	SPI_WriteData(data);
}

typedef struct {
    unsigned char type;  /* 0: cmd, 1: data */
    unsigned char para;
} LCM_setting_table;

static LCM_setting_table init_sequence[] =
{
	{0, 0x11},

	{0, 0x36},
	{1, 0xA0},

	{0, 0x3A},
	{1, 0x66},  //262k

	{0, 0xD2},  //enable vap/van output
	{1, 0x4C},

	{0, 0xB0},
	{1, 0x11},
	{1, 0xF0},

	{0, 0xB1},
	{1, 0xC0},
	{1, 0x02},
	{1, 0x14},

	{0, 0xB2},
	{1, 0x0C},  //back porch
	{1, 0x0C},  //front porch
	{1, 0x00},  //disable separate porch control
	{1, 0x33},
	{1, 0x33},

	{0, 0xB7},  //vgh/vgl
	{1, 0x35},

	{0, 0xBB},  //vcom
	{1, 0x1B},

	{0, 0xC0},
	{1, 0x2C},

	{0, 0xC2},
	{1, 0x01},

	{0, 0xC3},  //gamma
	{1, 0x20},

	{0, 0xC4},
	{1, 0x20},

	{0, 0xC6},  //frame rate
	{1, 0x0F},

	{0, 0xD0},
	{1, 0xA4},
	{1, 0xA1},

	{0, 0xE4},
	{1, 0x27},
	{1, 0x00},
	{1, 0x00},

	{0, 0xE0},
	{1, 0xD0},
	{1, 0x00},
	{1, 0x14},
	{1, 0x15},
	{1, 0x13},
	{1, 0x2C},
	{1, 0x42},
	{1, 0x43},
	{1, 0x4E},
	{1, 0x09},
	{1, 0x16},
	{1, 0x14},
	{1, 0x18},
	{1, 0x21},

	{0, 0xE1},
	{1, 0xD0},
	{1, 0x00},
	{1, 0x14},
	{1, 0x15},
	{1, 0x13},
	{1, 0x0B},
	{1, 0x43},
	{1, 0x55},
	{1, 0x53},
	{1, 0x0C},
	{1, 0x17},
	{1, 0x14},
	{1, 0x23},
	{1, 0x20},

	{0, 0xE9},
	{1, 0x11},
	{1, 0x11},
	{1, 0x03},

	{0, 0x20},
	{0, 0x29}
};

void ST7789V_TM2_4_RGB18b_init(void)
{
	int sequence_line, i;
	unsigned char cmd, data;

	sequence_line = sizeof(init_sequence) / sizeof(init_sequence[0]);

	for(i = 0; i < sequence_line; i++)
	{
		if (init_sequence[i].type == 0) {
			cmd = init_sequence[i].para;
			SPI_WRITE_CMD(cmd);
		} else {
			data = init_sequence[i].para;
			SPI_WRITE_DATA(data);
		}

		if(init_sequence[i].type == 0 && init_sequence[i].para == 0x11)
			MDELAY(5);
	}
	MDELAY(120);
}

static void ST7789V_TM2_4_RGB18b_sleep_in(void)
{
	SPI_WRITE_CMD(0x10);
	MDELAY(10);
}


static void ST7789V_TM2_4_RGB18b_sleep_out(void)
{
	SPI_WRITE_CMD(0x11);
	MDELAY(120);
}

/* --------------------------------------------------------------------------- */
/* Local Functions */
/* --------------------------------------------------------------------------- */
static void lcm_init_power(void)
{
#ifdef BUILD_LK
	printf("[LK/LCM] lcm_init_power() enter\n");
	lcm_set_gpio_output(GPIO_LCD_PWR, GPIO_OUT_ONE);
	MDELAY(20);
	upmu_set_rg_vgp1_vosel(3);
	upmu_set_rg_vgp1_en(0x1);

#else
	MDELAY(20);
	lcm_vgp_supply_enable();
	MDELAY(20);
	pr_err("[Kernel/LCM] lcm_init_power() enter\n");
#endif

}

static void lcm_suspend_power(void)
{
#ifdef BUILD_LK
	printf("[LK/LCM] lcm_suspend_power() enter\n");
	lcm_set_gpio_output(GPIO_LCD_PWR, GPIO_OUT_ZERO);
	MDELAY(20);

	upmu_set_rg_vgp1_vosel(0);
	upmu_set_rg_vgp1_en(0);

#else
	pr_err("[Kernel/LCM] lcm_suspend_power() enter\n");
	MDELAY(20);

	lcm_vgp_supply_disable();
	MDELAY(20);

#endif
}

static void lcm_resume_power(void)
{
#ifdef BUILD_LK
	printf("[LK/LCM] lcm_resume_power() enter\n");
	lcm_set_gpio_output(GPIO_LCD_PWR, GPIO_OUT_ONE);
	MDELAY(20);
	upmu_set_rg_vgp1_vosel(3);
	upmu_set_rg_vgp1_en(0x1);

#else
	pr_err("[Kernel/LCM] lcm_resume_power() enter\n");
	UDELAY(20);

	lcm_vgp_supply_enable();
	UDELAY(20);

#endif
}

/* --------------------------------------------------------------------------- */
/* LCM Driver Implementations */
/* --------------------------------------------------------------------------- */

static void lcm_set_util_funcs(const LCM_UTIL_FUNCS *util)
{
	memcpy(&lcm_util, util, sizeof(LCM_UTIL_FUNCS));
}

static void lcm_get_params(LCM_PARAMS *params)
{
	memset(params, 0, sizeof(LCM_PARAMS));

	params->type = LCM_TYPE_DPI;
	params->width = FRAME_WIDTH;
	params->height = FRAME_HEIGHT;

	params->physical_width = 36;
	params->physical_height = 48;

	params->dpi.PLL_CLOCK = 7;

	params->dpi.width = FRAME_WIDTH;
	params->dpi.height = FRAME_HEIGHT;

	params->dpi.clk_pol = LCM_POLARITY_FALLING;
	params->dpi.de_pol = LCM_POLARITY_RISING;
	params->dpi.vsync_pol = LCM_POLARITY_FALLING;
	params->dpi.hsync_pol = LCM_POLARITY_FALLING;

	params->dpi.hsync_pulse_width = HSYNC_PULSE_WIDTH;
	params->dpi.hsync_back_porch = HSYNC_BACK_PORCH;
	params->dpi.hsync_front_porch = HSYNC_FRONT_PORCH;
	params->dpi.vsync_pulse_width = VSYNC_PULSE_WIDTH;
	params->dpi.vsync_back_porch = VSYNC_BACK_PORCH;
	params->dpi.vsync_front_porch = VSYNC_FRONT_PORCH;

	params->dpi.lvds_tx_en = 0;
	params->dpi.format = LCM_DPI_FORMAT_RGB666;
	params->dpi.rgb_order = LCM_COLOR_ORDER_RGB;

	params->dpi.io_driving_current = LCM_DRIVING_CURRENT_32MA;
}

static void lcm_init_lcm(void)
{
#ifdef BUILD_LK
	printf("[LK/LCM] lcm_init() enter\n");

	lcm_set_gpio_output(GPIO_LCD_RST, 1);
	MDELAY(20);

	lcm_set_gpio_output(GPIO_LCD_RST, 0);
	MDELAY(20);

	lcm_set_gpio_output(GPIO_LCD_RST, 1);
	MDELAY(20);

	ST7789V_TM2_4_RGB18b_init();
#else
	pr_err("[Kernel/LCM] lcm_init() enter\n");

	lcm_get_gpio_infor();

#endif
}

void lcm_suspend(void)
{

#ifdef BUILD_LK
	printf("[LK/LCM] lcm_suspend() enter\n");
#else
	pr_err("[Kernel/LCM] lcm_suspend() enter\n");
#endif

	ST7789V_TM2_4_RGB18b_sleep_in();
}

void lcm_resume(void)
{
#ifdef BUILD_LK
	printf("[LK/LCM] lcm_resume() enter\n");
#else
	pr_err("[Kernel/LCM] lcm_resume() enter\n");
#endif

	ST7789V_TM2_4_RGB18b_sleep_out();
}

LCM_DRIVER t24bmhj201b_qvga_dpi_lcm_drv = {
	.name = "t24bmhj201b_qvga_dpi",
	.set_util_funcs = lcm_set_util_funcs,
	.get_params = lcm_get_params,
	.init = lcm_init_lcm,
	.suspend = lcm_suspend,
	.resume = lcm_resume,
	.init_power = lcm_init_power,
	.resume_power = lcm_resume_power,
	.suspend_power = lcm_suspend_power,
};

